@@ -84,6 +84,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
   private final int connectionTtl;
   private final int maxConnections;
   private final int maxConnectionsPerHost;
+  private final int acquireFreeChannelTimeout;
   private final ChannelPool channelPool;
   private final ConnectionSemaphoreFactory connectionSemaphoreFactory;
   private final KeepAliveStrategy keepAliveStrategy;
@@ -122,6 +123,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
   private final ByteBufAllocator allocator;
   private final boolean tcpNoDelay;
   private final boolean soReuseAddress;
+  private final boolean soKeepAlive;
   private final int soLinger;
   private final int soSndBuf;
   private final int soRcvBuf;
@@ -163,6 +165,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
                                        int connectionTtl,
                                        int maxConnections,
                                        int maxConnectionsPerHost,
+                                       int acquireFreeChannelTimeout,
                                        ChannelPool channelPool,
                                        ConnectionSemaphoreFactory connectionSemaphoreFactory,
                                        KeepAliveStrategy keepAliveStrategy,
@@ -191,6 +194,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
                                        // tuning
                                        boolean tcpNoDelay,
                                        boolean soReuseAddress,
+                                       boolean soKeepAlive,
                                        int soLinger,
                                        int soSndBuf,
                                        int soRcvBuf,
@@ -250,6 +254,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     this.connectionTtl = connectionTtl;
     this.maxConnections = maxConnections;
     this.maxConnectionsPerHost = maxConnectionsPerHost;
+    this.acquireFreeChannelTimeout = acquireFreeChannelTimeout;
     this.channelPool = channelPool;
     this.connectionSemaphoreFactory = connectionSemaphoreFactory;
     this.keepAliveStrategy = keepAliveStrategy;
@@ -278,6 +283,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     // tuning
     this.tcpNoDelay = tcpNoDelay;
     this.soReuseAddress = soReuseAddress;
+    this.soKeepAlive = soKeepAlive;
     this.soLinger = soLinger;
     this.soSndBuf = soSndBuf;
     this.soRcvBuf = soRcvBuf;
@@ -445,6 +451,9 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     return maxConnectionsPerHost;
   }
 
+  @Override
+  public int getAcquireFreeChannelTimeout() { return acquireFreeChannelTimeout; }
+
   @Override
   public ChannelPool getChannelPool() {
     return channelPool;
@@ -554,6 +563,11 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     return soReuseAddress;
   }
 
+  @Override
+  public boolean isSoKeepAlive() {
+    return soKeepAlive;
+  }
+
   @Override
   public int getSoLinger() {
     return soLinger;
@@ -696,6 +710,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     private int connectionTtl = defaultConnectionTtl();
     private int maxConnections = defaultMaxConnections();
     private int maxConnectionsPerHost = defaultMaxConnectionsPerHost();
+    private int acquireFreeChannelTimeout = defaultAcquireFreeChannelTimeout();
     private ChannelPool channelPool;
     private ConnectionSemaphoreFactory connectionSemaphoreFactory;
     private KeepAliveStrategy keepAliveStrategy = new DefaultKeepAliveStrategy();
@@ -719,6 +734,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
     // tuning
     private boolean tcpNoDelay = defaultTcpNoDelay();
     private boolean soReuseAddress = defaultSoReuseAddress();
+    private boolean soKeepAlive = defaultSoKeepAlive();
     private int soLinger = defaultSoLinger();
     private int soSndBuf = defaultSoSndBuf();
     private int soRcvBuf = defaultSoRcvBuf();
@@ -801,6 +817,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
       // tuning
       tcpNoDelay = config.isTcpNoDelay();
       soReuseAddress = config.isSoReuseAddress();
+      soKeepAlive = config.isSoKeepAlive();
       soLinger = config.getSoLinger();
       soSndBuf = config.getSoSndBuf();
       soRcvBuf = config.getSoRcvBuf();
@@ -991,6 +1008,16 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
       return this;
     }
 
+    /**
+     * Sets the maximum duration in milliseconds to acquire a free channel to send a request
+     * @param acquireFreeChannelTimeout maximum duration in milliseconds to acquire a free channel to send a request
+     * @return the same builder instance
+     */
+    public Builder setAcquireFreeChannelTimeout(int acquireFreeChannelTimeout) {
+      this.acquireFreeChannelTimeout = acquireFreeChannelTimeout;
+      return this;
+    }
+
     public Builder setChannelPool(ChannelPool channelPool) {
       this.channelPool = channelPool;
       return this;
@@ -1110,6 +1137,11 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
       return this;
     }
 
+    public Builder setSoKeepAlive(boolean soKeepAlive) {
+      this.soKeepAlive = soKeepAlive;
+      return this;
+    }
+
     public Builder setSoLinger(int soLinger) {
       this.soLinger = soLinger;
       return this;
@@ -1249,6 +1281,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
               connectionTtl,
               maxConnections,
               maxConnectionsPerHost,
+              acquireFreeChannelTimeout,
               channelPool,
               connectionSemaphoreFactory,
               keepAliveStrategy,
@@ -1269,6 +1302,7 @@ public class DefaultAsyncHttpClientConfig implements AsyncHttpClientConfig {
               cookieStore,
               tcpNoDelay,
               soReuseAddress,
+              soKeepAlive,
               soLinger,
               soSndBuf,
               soRcvBuf,
